/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_GEOMETRYLINESTRUCT_H
#define DPC_IMPL_TYPE_GEOMETRYLINESTRUCT_H

#include <cfloat>
#include <cmath>
#include "impl_type_boolean.h"
#include "dpc/impl_type_curveelementstruct.h"
#include "impl_type_uint16.h"
#include "dpc/impl_type_vector_samplingpointstruct.h"
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "impl_type_uint64.h"

namespace dpc {
struct GeometryLineStruct {
    ::Boolean is_curve_valid{ false };
    ::dpc::CurveElementStruct curve;
    ::Boolean is_sampling_point_valid{ false };
    ::UInt16 number_of_sampling_point{ 0U };
    ::dpc::vector_SamplingPointStruct sampling_point_list;
    ::UInt8 measurement_state{ 0U };
    ::Float life_time{ 0.0F };
    ::UInt64 last_measured_time_stamp{ 0U };
    ::Float exist_probability{ 0.0F };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(is_curve_valid);
        fun(curve);
        fun(is_sampling_point_valid);
        fun(number_of_sampling_point);
        fun(sampling_point_list);
        fun(measurement_state);
        fun(life_time);
        fun(last_measured_time_stamp);
        fun(exist_probability);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(is_curve_valid);
        fun(curve);
        fun(is_sampling_point_valid);
        fun(number_of_sampling_point);
        fun(sampling_point_list);
        fun(measurement_state);
        fun(life_time);
        fun(last_measured_time_stamp);
        fun(exist_probability);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("is_curve_valid", is_curve_valid);
        fun("curve", curve);
        fun("is_sampling_point_valid", is_sampling_point_valid);
        fun("number_of_sampling_point", number_of_sampling_point);
        fun("sampling_point_list", sampling_point_list);
        fun("measurement_state", measurement_state);
        fun("life_time", life_time);
        fun("last_measured_time_stamp", last_measured_time_stamp);
        fun("exist_probability", exist_probability);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("is_curve_valid", is_curve_valid);
        fun("curve", curve);
        fun("is_sampling_point_valid", is_sampling_point_valid);
        fun("number_of_sampling_point", number_of_sampling_point);
        fun("sampling_point_list", sampling_point_list);
        fun("measurement_state", measurement_state);
        fun("life_time", life_time);
        fun("last_measured_time_stamp", last_measured_time_stamp);
        fun("exist_probability", exist_probability);
    }

    friend bool operator==(const ::dpc::GeometryLineStruct& lhs, const ::dpc::GeometryLineStruct& rhs) noexcept
    {
        return (lhs.is_curve_valid == rhs.is_curve_valid) && (lhs.curve == rhs.curve) && (lhs.is_sampling_point_valid == rhs.is_sampling_point_valid) && (lhs.number_of_sampling_point == rhs.number_of_sampling_point) && (lhs.sampling_point_list == rhs.sampling_point_list) && (lhs.measurement_state == rhs.measurement_state) && (fabs(static_cast<double>(lhs.life_time - rhs.life_time)) < DBL_EPSILON) && (lhs.last_measured_time_stamp == rhs.last_measured_time_stamp) && (fabs(static_cast<double>(lhs.exist_probability - rhs.exist_probability)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_GEOMETRYLINESTRUCT_H
